#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from uuid import uuid4

# import interfaces
from pyams_content.interfaces import CREATE_CONTENT_PERMISSION, MANAGE_CONTENT_PERMISSION
from pyams_content.shared.site.interfaces import ISiteContainer, IContentLink
from pyams_content.skin.zmi.interfaces import ISiteTreeTable
from pyams_i18n.interfaces import II18n
from pyams_sequence.interfaces import IInternalReference
from pyams_skin.interfaces.container import ITableElementName
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_zmi.layer import IAdminLayer
from zope.intid.interfaces import IIntIds

# import packages
from pyams_content.shared.site.link import ContentLink
from pyams_content.shared.site.zmi.container import SiteContainerTreeTable, SiteContainerTreeNameColumn, \
    SiteContainerTreeSequenceColumn
from pyams_content.shared.site.zmi.widget import SiteManagerFoldersSelectorFieldWidget
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.table import get_object_name
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.registry import get_utility
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field
from zope.interface import Interface
from zope.schema import Int

from pyams_content import _


@viewlet_config(name='add-content-link.menu', context=ISiteContainer, layer=IAdminLayer, view=Interface,
                manager=IToolbarAddingMenu, permission=CREATE_CONTENT_PERMISSION, weight=90)
class ContentLinkAddMenu(ToolbarMenuItem):
    """Content link add menu"""

    label = _("Rent content...")
    label_css_class = 'fa fa-fw fa-external-link-square fa-rotate-90'
    url = 'add-content-link.html'
    modal_target = True


class IContentLinkAddFormFields(IContentLink):
    """Content link add forms fields interface"""

    parent = Int(title=_("Parent"),
                 description=_("Folder's parent"),
                 required=True)


@pagelet_config(name='add-content-link.html', context=ISiteContainer, layer=IPyAMSLayer,
                permission=CREATE_CONTENT_PERMISSION)
class ContentLinkAddForm(AdminDialogAddForm):
    """Content link add form"""

    legend = _("Rent existing content")

    fields = field.Fields(IContentLinkAddFormFields).select('reference', 'alt_title', 'parent')
    fields['parent'].widgetFactory = SiteManagerFoldersSelectorFieldWidget

    ajax_handler = 'add-content-link.json'
    edit_permission = CREATE_CONTENT_PERMISSION

    __target = None

    def updateWidgets(self, prefix=None):
        super(ContentLinkAddForm, self).updateWidgets(prefix)
        if 'parent' in self.widgets:
            self.widgets['parent'].permission = CREATE_CONTENT_PERMISSION

    def create(self, data):
        return ContentLink()

    def update_content(self, content, data):
        content.reference = data.get('reference')
        content.alt_title = data['alt_title']
        intids = get_utility(IIntIds)
        parent = intids.queryObject(data.get('parent'))
        if parent is not None:
            uuid = str(uuid4())
            parent[uuid] = content
            self.__target = parent

    def add(self, content):
        pass

    def nextURL(self):
        return absolute_url(self.__target, self.request, 'admin#site-tree.html')


@view_config(name='add-content-link.json', context=ISiteContainer, request_type=IPyAMSLayer,
             permission=CREATE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ContentLinkAJAXAddForm(AJAXAddForm, ContentLinkAddForm):
    """Content link add form, JSOn renderer"""


@adapter_config(context=(IContentLink, IPyAMSLayer, ISiteTreeTable), provides=ITableElementName)
class ContentLinkTableElementName(ContextRequestViewAdapter):
    """Content link table element name"""

    @property
    def name(self):
        title = II18n(self.context).query_attribute('alt_title', request=self.request)
        if not title:
            target = self.context.get_target()
            if target is not None:
                title = get_object_name(target, self.request, self.view)
        return '<i class="fa fa-fw fa-external-link-square fa-rotate-90"></i>{title}'.format(
            title=title or '--')


@pagelet_config(name='properties.html', context=IContentLink, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class ContentLinkPropertiesEditForm(AdminDialogEditForm):
    """Content link properties edit form"""

    legend = _("Edit content link properties")

    fields = field.Fields(IContentLink).omit('__parent__', '__name__')

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION


@view_config(name='properties.json', context=IContentLink, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ContentLinkPropertiesAJAXEditForm(AJAXEditForm, ContentLinkPropertiesEditForm):
    """Content link properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(ContentLinkPropertiesAJAXEditForm, self).get_ajax_output(changes)
        intids = get_utility(IIntIds)
        if 'reference' in changes.get(IInternalReference, ()):
            table = SiteContainerTreeTable(self.context.__parent__, self.request)
            table.update()
            row = table.setUpRow(self.context)
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'MyAMS.skin.refreshRow',
                    'object_id': '{0}::{1}'.format(SiteContainerTreeTable.id, intids.queryId(self.context)),
                    'row': table.renderRow(row)
                }
            })
        elif 'alt_title' in changes.get(IContentLink, ()):
            adapter = ContentLinkTableElementName(self.context, self.request, None)
            column = SiteContainerTreeNameColumn(self.context, self.request, None)
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'MyAMS.skin.refreshRowCell',
                    'object_id': '{0}::{1}'.format(SiteContainerTreeTable.id, intids.queryId(self.context)),
                    'col_name': 'name',
                    'cell': column.renderCell(self.context, name=adapter.name)
                }
            })
        return output
